//
//  Strip.h
//  iStrip
//
//  Created by Joar Wingfors on Sat Dec 14 2002.
//  Copyright (c) 2002 joar.com. All rights reserved.
//

#import "StripProtocol.h"

@interface StripValueTransformer : NSObject

// Transforming strip values <-> dates
+ (NSCalendarDate *) dateForValue:(StripValue) value;
+ (StripValue) valueForDate:(NSCalendarDate *) date;

// Transforming strip values <-> dayOfCommonEra
+ (unsigned int) dayOfCommonEraForValue:(StripValue) value;
+ (StripValue) valueForDayOfCommonEra:(unsigned int) day;

// Transforming strip values <-> indexes
+ (unsigned int) indexForValue:(StripValue) value;
+ (StripValue) valueForIndex:(unsigned int) index;

// Marshalling of strip values
+ (id) plistForStripValue:(StripValue) value;
+ (StripValue) stripValueForPlist:(id) pList;

@end

@class HTTPConnection, PreloadController;

@interface Strip : NSObject < StripProtocol, StripOptionalMethodsProtocol >
{
	@private
	
	NSString *myStripId;
	NSString *myStripName;
	
	BOOL myIsEnabledFlag;
	BOOL myIsBusyFlag;
	
	HTTPConnection *myHTTPConnection;
	PreloadController *myPreloadController;
	
	BOOL myIsDeterminatePreloadProgressFlag;
	unsigned int myCurrentPreloadStep;
	unsigned int myTotalPreloadStepCount;
	unsigned int myCurrentDownloadStep;
	unsigned int myTotalDownloadStepCount;
	
	id < StripStateDelegate > myStateDelegate;
	id < StripPreloadDelegate > myPreloadDelegate;
	id < StripImageDataDownloadDelegate > myDownloadDelegate;
	
	StripValue myDownloadValue;
}

+ (id) stripWithId:(NSString *) stripId name:(NSString *) stripName;
- (id) initWithStripId:(NSString *) stripId name:(NSString *) stripName;

@end

@protocol PreloadControllerDelegate, HTTPConnectionDelegate;

extern NSString * const kPage_RequestKey;
extern NSString * const kImage_RequestKey;

@interface Strip (Protected) < PreloadControllerDelegate, HTTPConnectionDelegate >

+ (Class) preloadControllerClass; // Override in subclasses to return the appropriate preload controller class

- (BOOL) isUsingPreload; // Defaults to YES, override to change behaviour

- (PreloadController *) preloadController;
- (void) willPreload;
- (NSURL *) preloadURL; // Required: Override in subclasses to return page to preload
- (void) preloadFromURL:(NSURL *) url; // Call to request download of page
- (void) handlePreloadedPage:(NSString *) html; // Required: Override in subclasses to scrub data from downloaded page

- (void) loadFromURL:(NSURL *) url withRequestKey:(NSString *) key; // Call to request download of HTML

// Update with correct values for the current download operation, or 1 if unknown. All values default to 1.
// Override getters for static values in subclasses.
- (BOOL) isDeterminatePreloadProgress;
- (void) setIsDeterminatePreloadProgress:(BOOL) determinate;
- (unsigned int) currentPreloadStep;
- (void) setCurrentPreloadStep:(unsigned int) step;
- (unsigned int) totalPreloadStepCount;
- (void) setTotalPreloadStepCount:(unsigned int) count;
- (unsigned int) currentDownloadStep;
- (void) setCurrentDownloadStep:(unsigned int) step;
- (unsigned int) totalDownloadStepCount;
- (void) setTotalDownloadStepCount:(unsigned int) count;

// Required: Override in subclasses to return a corresponding URL
- (NSURL *) URLForDownloadValue:(StripValue) value htmlString:(NSString *) htmlStr withRequestKey:(NSString **) key;

@end

@interface Strip (Defaults)

- (id) persistedObjectForKey:(NSString *) defaultName;
- (void) persistObject:(id) value forKey:(NSString *) defaultName;
- (void) removePersistedObjectForKey:(NSString *) defaultName;

@end

typedef enum _StripHasImage
{
	StripHasImage_No,
	StripHasImage_Yes,
	StripHasImage_Maybe
} StripHasImage;

@interface DateStrip : Strip

- (DatePreloadController *) datePreloadController;

// All methods below this point requires overrides in subclasses
- (NSURL *) URLForDate:(NSCalendarDate *) date htmlString:(NSString *) htmlStr withRequestKey:(NSString **) key;
- (NSURL *) webURLForDate:(NSCalendarDate *) date;

@end

@interface IndexStrip : Strip

- (IndexPreloadController *) indexPreloadController;

// All methods below this point requires overrides in subclasses
- (NSURL *) URLForIndex:(unsigned int) index htmlString:(NSString *) htmlStr withRequestKey:(NSString **) key;
- (NSURL *) webURLForIndex:(unsigned int) index;

@end

@interface NoPreloadDateStrip : DateStrip
{
    @private
	
	NSCalendarDate *myFirstDate;
}

+ (id) stripWithId:(NSString *) stripId name:(NSString *) stripName firstDate:(NSCalendarDate *) date;

- (NSCalendarDate *) dateForFirstStripInArchive; // Default to myFirstDate
- (NSCalendarDate *) dateForLastStripInArchive; // Default to today
- (StripHasImage) hasStripForDate:(NSCalendarDate *) date; // Default to StripHasImage_Maybe

@end

@interface NoPreloadIndexStrip : IndexStrip
{
    @private
	
	unsigned int myFirstIndex;
	unsigned int myLastIndex;
}

+ (id) stripWithId:(NSString *) stripId name:(NSString *) stripName firstIndex:(unsigned int) firstIndex lastIndex:(unsigned int) lastIndex;

- (unsigned int) indexForFirstStripInArchive; // Default to myFirstIndex
- (unsigned int) indexForLastStripInArchive; // Default to myLastIndex
- (StripHasImage) hasStripForIndex:(unsigned int) index; // Default to StripHasImage_Maybe

@end

@interface RetiredDateStrip : DateStrip
{
    @private
	
	NSString *myLanguage;
}

+ (id) stripWithId:(NSString *) stripId name:(NSString *) stripName language:(NSString *) stripLanguage;

@end

@interface RetiredIndexStrip : IndexStrip
{
    @private
	
	NSString *myLanguage;
}

+ (id) stripWithId:(NSString *) stripId name:(NSString *) stripName language:(NSString *) stripLanguage;

@end
